# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from ..commons.types.space_id import SpaceId
import typing
from ..core.request_options import RequestOptions
from .types.sftp_credentials import SftpCredentials
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.types.errors import Errors
from ..commons.errors.not_found_error import NotFoundError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper


class AuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_sftp_credentials(
        self, *, space_id: SpaceId, request_options: typing.Optional[RequestOptions] = None
    ) -> SftpCredentials:
        """
        Parameters
        ----------
        space_id : SpaceId
            ID of space to get credentials for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SftpCredentials

        Examples
        --------
        from flatfile import Flatfile

        client = Flatfile(
            token="YOUR_TOKEN",
        )
        client.auth.get_sftp_credentials(
            space_id="spaceId",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "auth/sftp-credentials",
            method="GET",
            params={
                "spaceId": space_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SftpCredentials,
                    parse_obj_as(
                        type_=SftpCredentials,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Errors,
                        parse_obj_as(
                            type_=Errors,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Errors,
                        parse_obj_as(
                            type_=Errors,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_sftp_credentials(
        self, *, space_id: SpaceId, request_options: typing.Optional[RequestOptions] = None
    ) -> SftpCredentials:
        """
        Parameters
        ----------
        space_id : SpaceId
            ID of space to get credentials for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SftpCredentials

        Examples
        --------
        import asyncio

        from flatfile import AsyncFlatfile

        client = AsyncFlatfile(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.auth.get_sftp_credentials(
                space_id="spaceId",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "auth/sftp-credentials",
            method="GET",
            params={
                "spaceId": space_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SftpCredentials,
                    parse_obj_as(
                        type_=SftpCredentials,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Errors,
                        parse_obj_as(
                            type_=Errors,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Errors,
                        parse_obj_as(
                            type_=Errors,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
